@@ -8,6 +8,7 @@ use {
         errors::ProgramError,
         file_sum, git,
         launchable::Launchable,
+        path::closest_dir,
         skin::*,
         task_sync::{Dam, Either},
         verb::Internal,
@@ -26,8 +27,6 @@ use {
     termimad::{Event, EventSource},
 };
 
-const ESCAPE_TO_QUIT: bool = false;
-
 #[cfg(feature = "client-server")]
 use std::sync::{Arc, Mutex};
 
@@ -346,47 +345,95 @@ impl App {
                 self.tx_seqs.send(sequence).unwrap();
             }
             HandleInApp(internal) => {
-                let new_active_panel_idx = match internal {
-                    Internal::panel_left => {
-                        // we're not here to create a panel, this is handled in the state
-                        // we're here because the state wants us to either move to the panel
-                        // to the left, or close the rightest one
-                        if self.active_panel_idx == 0 {
-                            self.close_panel(self.panels.len().get()-1);
-                            None
-                        } else {
-                            Some(self.active_panel_idx - 1)
+                match internal {
+                    Internal::panel_left | Internal::panel_right => {
+                        let new_active_panel_idx = if internal == Internal::panel_left {
+                            // we're not here to create a panel, this is handled in the state
+                            // we're here because the state wants us to either move to the panel
+                            // to the left, or close the rightest one
+                            if self.active_panel_idx == 0 {
+                                self.close_panel(self.panels.len().get()-1);
+                                None
+                            } else {
+                                Some(self.active_panel_idx - 1)
+                            }
+                        } else { // panel_right
+                            // we're not here to create panels (it's done in the state).
+                            // So we either move to the right or close the leftes panel
+                            if self.active_panel_idx + 1 == self.panels.len().get() {
+                                self.close_panel(0);
+                                None
+                            } else {
+                                Some(self.active_panel_idx + 1)
+                            }
+                        };
+                        if let Some(idx) = new_active_panel_idx {
+                            if is_input_invocation {
+                                self.mut_panel().clear_input();
+                            }
+                            self.active_panel_idx = idx;
+                            let app_cmd_context = AppCmdContext {
+                                other_path: self.get_other_panel_path(),
+                                panel_skin,
+                                preview_panel: self.preview_panel,
+                                stage_panel: self.stage_panel,
+                                screen: self.screen,
+                                con,
+                            };
+                            self.mut_panel().refresh_input_status(app_state, &app_cmd_context);
                         }
                     }
-                    Internal::panel_right => {
-                        // we're not here to create panels (it's done in the state).
-                        // So we either move to the right or close the leftes panel
-                        if self.active_panel_idx + 1 == self.panels.len().get() {
-                            self.close_panel(0);
-                            None
+                    Internal::toggle_second_tree => {
+                        let panels_count = self.panels.len().get();
+                        let trees_count = self.panels.iter()
+                            .filter(|p| p.state().get_type() == PanelStateType::Tree)
+                            .count();
+                        if trees_count < 2 {
+                            // we open a tree, closing a (non tree) panel if necessary
+                            if panels_count >= con.max_panels_count {
+                                for i in (0..panels_count).rev() {
+                                    if self.panels[i].state().get_type() != PanelStateType::Tree {
+                                        self.close_panel(i);
+                                        break;
+                                    }
+                                }
+                            }
+                            if let Some(selected_path) = self.state().selected_path() {
+                                let dir = closest_dir(selected_path);
+                                if let Ok(Some(new_state)) = BrowserState::new(
+                                    dir,
+                                    self.state().tree_options().without_pattern(),
+                                    self.screen,
+                                    con,
+                                    &Dam::unlimited(),
+                                ) {
+                                    if let Err(s) = self.new_panel(
+                                        Box::new(new_state),
+                                        PanelPurpose::None,
+                                        HDir::Right,
+                                        is_input_invocation,
+                                        con,
+                                    ) {
+                                        error = Some(s);
+                                    }
+                                }
+                            }
                         } else {
-                            Some(self.active_panel_idx + 1)
+                            // we close the rightest inactive tree
+                            for i in (0..panels_count).rev() {
+                                if self.panels[i].state().get_type() == PanelStateType::Tree {
+                                    if i == self.active_panel_idx {
+                                        continue;
+                                    }
+                                    self.close_panel(i);
+                                    break;
+                                }
+                            }
                         }
                     }
                     _ => {
-                        debug!("unhandled propagated internal. cmd={:?}", &cmd);
-                        None
+                        info!("unhandled propagated internal. cmd={:?}", &cmd);
                     }
-                };
-                if let Some(idx) = new_active_panel_idx {
-                    if is_input_invocation {
-                        self.mut_panel().clear_input();
-                    }
-                    self.active_panel_idx = idx;
-                    let app_cmd_context = AppCmdContext {
-                        other_path: self.get_other_panel_path(),
-                        panel_skin,
-                        preview_panel: self.preview_panel,
-                        stage_panel: self.stage_panel,
-                        screen: self.screen,
-                        con,
-                    };
-                    self.mut_panel().refresh_input_status(app_state, &app_cmd_context);
                 }
             }
             Keep => {
@@ -419,7 +466,7 @@ impl App {
                 if self.remove_state() {
                     self.mut_state().refresh(app_cmd_context.screen, con);
                     self.mut_panel().refresh_input_status(app_state, &app_cmd_context);
-                } else if ESCAPE_TO_QUIT {
+                } else if con.quit_on_last_cancel {
                     self.quitting = true;
                 }
             }
@@ -437,7 +484,7 @@ impl App {
                         con,
                     };
                     self.mut_panel().apply_command(w, &cmd, app_state, &app_cmd_context)?;
-                } else if ESCAPE_TO_QUIT {
+                } else if con.quit_on_last_cancel {
                     self.quitting = true;
                 }
             }
@@ -496,7 +543,7 @@ impl App {
         state: Box<dyn PanelState>,
         purpose: PanelPurpose,
         direction: HDir,
-        is_input_invocation: bool,
+        is_input_invocation: bool, // if true we clean the input
         con: &AppContext,
     ) -> Result<(), String> {
         match state.get_type() {
@@ -560,7 +607,7 @@ impl App {
         con: &AppContext,
     ) -> Result<(), ProgramError> {
         while self.has_pending_task() && !dam.has_event() {
-            if self.do_pending_task(con, dam) {
+            if self.do_pending_task(app_state, con, dam) {
                 self.update_preview(con); // the selection may have changed
                 let app_cmd_context = AppCmdContext {
                     other_path: self.get_other_panel_path(),
@@ -583,13 +630,14 @@ impl App {
     /// do the next pending task
     fn do_pending_task(
         &mut self,
+        app_state: &AppState,
         con: &AppContext,
         dam: &mut Dam,
     ) -> bool {
         let screen = self.screen;
         // we start with the focused panel
         if self.panel().has_pending_task() {
-            self.mut_panel().do_pending_task(screen, con, dam);
+            self.mut_panel().do_pending_task(&app_state.stage, screen, con, dam);
             return true;
         }
         // then the other ones
@@ -597,7 +645,7 @@ impl App {
             if idx != self.active_panel_idx {
                 let panel = &mut self.panels[idx];
                 if panel.has_pending_task() {
-                    panel.do_pending_task(screen, con, dam);
+                    panel.do_pending_task(&app_state.stage, screen, con, dam);
                     return true;
                 }
             }
@@ -645,6 +693,7 @@ impl App {
             if !self.quitting {
                 self.display_panels(w, &skin, &app_state, con)?;
                 time!(
+                    Info,
                     "pending_tasks",
                     self.do_pending_tasks(w, &skin, &mut dam, &mut app_state, con)?,
                 );
@@ -695,7 +744,7 @@ impl App {
                             // is that a 100% safe way of quitting ?
                             return Ok(self.launch_at_end.take());
                         } else {
-                            self.display_panels(w, &skin, &mut app_state, con)?;
+                            self.display_panels(w, &skin, &app_state, con)?;
                             time!(
                                 "sequence pending tasks",
                                 self.do_pending_tasks(w, &skin, &mut dam, &mut app_state, con)?,
