@@ -9,7 +9,7 @@ use {
         pattern::*,
         preview::{PreviewMode, PreviewState},
         print,
-        stage::StageState,
+        stage::*,
         task_sync::Dam,
         tree::*,
         verb::*,
@@ -302,6 +302,9 @@ pub trait PanelState {
             Internal::panel_right => {
                 CmdResult::HandleInApp(Internal::panel_right)
             }
+            Internal::toggle_second_tree => {
+                CmdResult::HandleInApp(Internal::toggle_second_tree)
+            }
             Internal::clear_stage => {
                 app_state.stage.clear();
                 if let Some(panel_id) = cc.app.stage_panel {
@@ -647,6 +650,7 @@ pub trait PanelState {
 
     fn do_pending_task(
         &mut self,
+        _stage: &Stage,
         _screen: Screen,
         _con: &AppContext,
         _dam: &mut Dam,
@@ -655,7 +659,9 @@ pub trait PanelState {
         unreachable!();
     }
 
-    fn get_pending_task(&self) -> Option<&'static str> {
+    fn get_pending_task(
+        &self,
+    ) -> Option<&'static str> {
         None
     }
 
